import { __assign } from "tslib";
import React, { useState, useRef } from 'react';
var svgIconPath = 'm25,1 6,17h18l-14,11 5,17-15-10-15,10 5-17-14-11h18z';
var svgIconViewBox = '0 0 51 48';
var FullStar = function (_a) {
    var ICON_STYLES = _a.ICON_STYLES, COLOR = _a.COLOR;
    return (React.createElement("div", { style: { color: COLOR } },
        React.createElement("svg", { viewBox: svgIconViewBox, style: ICON_STYLES },
            React.createElement("path", { d: svgIconPath, fill: 'currentColor' }))));
};
var EmptyStar = function (_a) {
    var ICON_STYLES = _a.ICON_STYLES, _b = _a.DEFAULT_COLOR, DEFAULT_COLOR = _b === void 0 ? 'rgb(203, 211, 227)' : _b, COLOR = _a.COLOR;
    var pathStyle = {
        fill: DEFAULT_COLOR,
        transition: 'fill .2s ease-in-out',
    };
    return (React.createElement("div", { style: { color: COLOR } },
        React.createElement("svg", { viewBox: svgIconViewBox, style: ICON_STYLES },
            React.createElement("path", { style: pathStyle, d: svgIconPath }))));
};
var HeartEmptyIcon = function (_a) {
    var ICON_STYLES = _a.ICON_STYLES, COLOR = _a.COLOR, DEFAULT_COLOR = _a.DEFAULT_COLOR;
    return (React.createElement("div", { style: { color: COLOR } },
        React.createElement("svg", { viewBox: '0 0 24 24', style: ICON_STYLES, fill: DEFAULT_COLOR },
            React.createElement("path", { d: 'M16.5 3c-1.74 0-3.41.81-4.5 2.09C10.91 3.81 9.24 3 7.5 3 4.42 3 2 5.42 2 8.5c0 3.78 3.4 6.86 8.55 11.54L12 21.35l1.45-1.32C18.6 15.36 22 12.28 22 8.5 22 5.42 19.58 3 16.5 3zm-4.4 15.55-.1.1-.1-.1C7.14 14.24 4 11.39 4 8.5 4 6.5 5.5 5 7.5 5c1.54 0 3.04.99 3.57 2.36h1.87C13.46 5.99 14.96 5 16.5 5c2 0 3.5 1.5 3.5 3.5 0 2.89-3.14 5.74-7.9 10.05z' }))));
};
var HeartFullIcon = function (_a) {
    var ICON_STYLES = _a.ICON_STYLES, COLOR = _a.COLOR;
    return (React.createElement("div", { style: { color: COLOR } },
        React.createElement("svg", { viewBox: '0 0 24 24', style: ICON_STYLES, fill: 'currentColor' },
            React.createElement("path", { d: 'm12 21.35-1.45-1.32C5.4 15.36 2 12.28 2 8.5 2 5.42 4.42 3 7.5 3c1.74 0 3.41.81 4.5 2.09C13.09 3.81 14.76 3 16.5 3 19.58 3 22 5.42 22 8.5c0 3.78-3.4 6.86-8.55 11.54L12 21.35z' }))));
};
var Rating = function (_a) {
    var _b = _a.precision, precision = _b === void 0 ? 1 : _b, _c = _a.count, count = _c === void 0 ? 5 : _c, _d = _a.shape, shape = _d === void 0 ? 'star' : _d, _e = _a.defaultValue, defaultValue = _e === void 0 ? 0 : _e, onChange = _a.onChange, onHover = _a.onHover, _f = _a.classNames, classNames = _f === void 0 ? '' : _f, _g = _a.readOnly, readOnly = _g === void 0 ? false : _g, _h = _a.size, size = _h === void 0 ? '24px' : _h, _j = _a.spacing, spacing = _j === void 0 ? '5px' : _j, _k = _a.activeColor, activeColor = _k === void 0 ? 'orange' : _k, _l = _a.defaultColor, defaultColor = _l === void 0 ? 'gray' : _l, _m = _a.titleArray, titleArray = _m === void 0 ? ['Poor', 'Good', 'Very Good', 'Best', 'Excellent'] : _m, _o = _a.showTitle, showTitle = _o === void 0 ? false : _o;
    if (showTitle && titleArray.length < count) {
        throw new Error('titleArray length must be greater than or equal to count.');
    }
    var _p = useState(defaultValue), activeIcon = _p[0], setActiveIcon = _p[1];
    var _q = useState(-1), hoverActiveIcon = _q[0], setHoverActiveIcon = _q[1];
    var _r = useState(false), isHovered = _r[0], setIsHovered = _r[1];
    var ratingContainerRef = useRef(null);
    var calculateRating = function (e) {
        var _a;
        var _b = ratingContainerRef.current.getBoundingClientRect(), width = _b.width, left = _b.left;
        var percent = (e.clientX - left) / width;
        var numberInStars = percent * count;
        var nearestNumber = Math.round((numberInStars + precision / 2) / precision) * precision;
        return Number(nearestNumber.toFixed(((_a = precision.toString().split('.')[1]) === null || _a === void 0 ? void 0 : _a.length) || 0));
    };
    var handleClick = function (e) {
        if (readOnly)
            return;
        setIsHovered(false);
        setActiveIcon(calculateRating(e));
        onChange && onChange(calculateRating(e));
    };
    var handleMouseMove = function (e) {
        if (readOnly)
            return;
        setIsHovered(true);
        setHoverActiveIcon(calculateRating(e));
        onHover && onHover(calculateRating(e));
    };
    var handleMouseLeave = function () {
        if (readOnly)
            return;
        setHoverActiveIcon(-1); // Reset to default state
        setIsHovered(false);
    };
    var ICON_STYLES = {
        width: size,
        height: size,
    };
    var IconProps = {
        ICON_STYLES: ICON_STYLES,
        COLOR: activeColor,
        DEFAULT_COLOR: defaultColor,
    };
    var getShape = function (style) {
        switch (shape) {
            case 'heart':
                if (style === 'full')
                    return React.createElement(HeartFullIcon, __assign({}, IconProps));
                return React.createElement(HeartEmptyIcon, __assign({}, IconProps));
            default:
                if (style === 'full')
                    return React.createElement(FullStar, __assign({}, IconProps));
                return React.createElement(EmptyStar, __assign({}, IconProps));
        }
    };
    return (React.createElement("div", { style: { display: 'flex', alignItems: 'center' } },
        React.createElement("div", { onClick: handleClick, onMouseMove: handleMouseMove, onMouseLeave: handleMouseLeave, ref: ratingContainerRef, className: "md-rating-container ".concat(classNames, " ").concat(readOnly ? 'md-rating-container--readonly' : ''), style: {
                display: 'flex',
                gap: spacing,
                position: 'relative',
                cursor: readOnly ? 'default' : 'pointer',
                lineHeight: 0,
                boxSizing: 'border-box',
                padding: 0,
                margin: 0,
            } }, new Array(count).fill('').map(function (_, index) {
            var activeState = isHovered ? hoverActiveIcon : activeIcon;
            var showEmptyIcon = activeState === -1 || activeState < index + 1;
            var isActiveRating = activeState !== 1;
            var isRatingWithPrecision = activeState % 1 !== 0;
            var isRatingEqualToIndex = Math.ceil(activeState) === index + 1;
            var showRatingWithPrecision = isActiveRating && isRatingWithPrecision && isRatingEqualToIndex;
            return (React.createElement("div", { className: 'md-rating-star', key: index, style: {
                    position: 'relative',
                    cursor: readOnly ? 'default' : 'pointer',
                } },
                React.createElement("div", { className: 'md-rating-star--full', style: {
                        width: showRatingWithPrecision ? "".concat((activeState % 1) * 100, "%") : '0%',
                        overflow: 'hidden',
                        position: 'absolute',
                    }, title: showTitle ? titleArray[index] : '' }, getShape('full')),
                React.createElement("div", null, showEmptyIcon ? getShape('empty') : getShape('full'))));
        }))));
};
export default Rating;
